/**
 * ButtonDebounced.h
 * 
 * Declaration of the class ButtonDebounced. The methods onClick (), onLongClick () 
 * and onDoubleKlick () are virtual and are implemented later in the derived class
 * (see class MenuControlButton).
 */
#ifndef _BUTTONDEBOUNCED_H_
#define _BUTTONDEBOUNCED_H_
#include <Arduino.h>

class ButtonDebounced
{
  public:
    ButtonDebounced(uint8_t pin, void (&onClick)(), void(&onLongClick)(), void (&onDoubleClick)()) : 
      _pin(pin), _onClick(onClick), _onLongClick(onLongClick), _onDoubleClick(onDoubleClick) 
      {
        pinMode(_pin, INPUT_PULLUP);
      }

    void setup();
    void loop();

  private:
    const uint8_t _pin;
    uint8_t  _clickCount;
    uint8_t  _btnState = HIGH;
    uint32_t _msDebounce = 50;        // After 50ms the button should have reached a steady state
    uint32_t _msLongClick = 300;      // A Button that is held longer than 300ms is considered as longClick
    uint32_t _msDoubleClickGap = 250; // Two button clicks within 250ms count as a doubleClick
    uint32_t _msButtonDown = 0;
    uint32_t _msFirstClick = 0;
    void (&_onClick)();
    void (&_onLongClick)();
    void (&_onDoubleClick)();

};
#endif